@@ -1,5 +1,3 @@
-# -*- coding:utf-8 -*-
-
 # Copyright (c) 2009-2018 - Simon Conseil
 # Copyright (c) 2013      - Christophe-Marie Duquesne
 # Copyright (c) 2014      - Jonas Kaufmann
@@ -25,12 +23,12 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
 
-from __future__ import absolute_import, print_function
-
 import fnmatch
+import locale
 import logging
 import multiprocessing
 import os
+import pickle
 import random
 import sys
 import zipfile
@@ -40,9 +38,9 @@ from collections import defaultdict
 from datetime import datetime
 from itertools import cycle
 from os.path import isfile, join, splitext
+from urllib.parse import quote as url_quote
 
 from . import image, video, signals
-from .compat import PY2, UnicodeMixin, strxfrm, url_quote, text_type, pickle
 from .image import (process_image, get_exif_tags, get_exif_data, get_size,
                     get_iptc_data)
 from .settings import get_thumb
@@ -50,10 +48,10 @@ from .utils import (Devnull, copy, check_or_create_dir, url_from_path,
                     read_markdown, cached_property, is_valid_html5_video,
                     get_mime)
 from .video import process_video
-from .writer import Writer
+from .writer import AlbumPageWriter, AlbumListPageWriter
 
 
-class Media(UnicodeMixin):
+class Media:
     """Base Class for media files.
 
     Attributes:
@@ -62,6 +60,7 @@ class Media(UnicodeMixin):
     - ``filename``: Filename of the resized image.
     - ``thumbnail``: Location of the corresponding thumbnail image.
     - ``big``: If not None, location of the unmodified image.
+    - ``big_url``: If not None, url of the unmodified image.
     - ``exif``: If not None contains a dict with the most common tags. For more
         information, see :ref:`simple-exif-data`.
     - ``raw_exif``: If not ``None``, it contains the raw EXIF tags.
@@ -71,7 +70,7 @@ class Media(UnicodeMixin):
     type = ''
 
     def __init__(self, filename, path, settings):
-        self.src_filename = self.filename = self.url = filename
+        self.src_filename = self.filename = filename
         self.path = path
         self.settings = settings
         self.ext = os.path.splitext(filename)[1].lower()
@@ -84,14 +83,22 @@ class Media(UnicodeMixin):
 
         self.logger = logging.getLogger(__name__)
         self._get_metadata()
+        # default: title is the filename
+        if not self.title:
+            self.title = self.filename
         signals.media_initialized.send(self)
 
     def __repr__(self):
         return "<%s>(%r)" % (self.__class__.__name__, str(self))
 
-    def __unicode__(self):
+    def __str__(self):
         return join(self.path, self.filename)
 
+    @property
+    def url(self):
+        """URL of the media."""
+        return url_from_path(self.filename)
+
     @property
     def big(self):
         """Path to the original image, if ``keep_orig`` is set (relative to the
@@ -106,9 +113,14 @@ class Media(UnicodeMixin):
             check_or_create_dir(orig_path)
             big_path = join(orig_path, self.src_filename)
             if not isfile(big_path):
-                copy(self.src_path, big_path,
-                     symlink=s['orig_link'])
-            return url_from_path(join(s['orig_dir'], self.src_filename))
+                copy(self.src_path, big_path, symlink=s['orig_link'])
+            return join(s['orig_dir'], self.src_filename)
+
+    @property
+    def big_url(self):
+        """URL of the original media."""
+        if self.big is not None:
+            return url_from_path(self.big)
 
     @property
     def thumbnail(self):
@@ -159,7 +171,8 @@ class Image(Media):
 
     @cached_property
     def date(self):
-        return self.exif and self.exif.get('dateobj', None) or self._get_file_date()
+        return (self.exif and self.exif.get('dateobj', None) or
+                self._get_file_date())
 
     @cached_property
     def exif(self):
@@ -175,11 +188,16 @@ class Image(Media):
             # Nothing to do - we already have title and description
             return
 
-        iptc_data = get_iptc_data(self.src_path)
-        if not self.title and iptc_data.get('title'):
-            self.title = iptc_data['title']
-        if not self.description and iptc_data.get('description'):
-            self.description = iptc_data['description']
+        try:
+            iptc_data = get_iptc_data(self.src_path)
+        except Exception as e:
+            self.logger.warning('Could not read IPTC data from %s: %s',
+                                self.src_path, e)
+        else:
+            if not self.title and iptc_data.get('title'):
+                self.title = iptc_data['title']
+            if not self.description and iptc_data.get('description'):
+                self.description = iptc_data['description']
 
     @cached_property
     def raw_exif(self):
@@ -187,7 +205,7 @@ class Image(Media):
             return (get_exif_data(self.src_path)
                     if self.ext in ('.jpg', '.jpeg') else None)
         except Exception as e:
-            self.logger.warning(u'Could not read EXIF data from %s: %s',
+            self.logger.warning('Could not read EXIF data from %s: %s',
                                 self.src_path, e)
 
     @cached_property
@@ -215,14 +233,14 @@ class Video(Media):
         if not settings['use_orig'] or not is_valid_html5_video(ext):
             video_format = settings['video_format']
             ext = '.' + video_format
-            self.filename = self.url = base + ext
+            self.filename = base + ext
             self.mime = get_mime(ext)
             self.dst_path = join(settings['destination'], path, base + ext)
         else:
             self.mime = get_mime(ext)
 
 
-class Album(UnicodeMixin):
+class Album:
     """Gather all informations on an album.
 
     Attributes:
@@ -289,9 +307,9 @@ class Album(UnicodeMixin):
         return "<%s>(path=%r, title=%r)" % (self.__class__.__name__, self.path,
                                             self.title)
 
-    def __unicode__(self):
-        return (u"{} : ".format(self.path) +
-                ', '.join("{} {}s".format(count, _type)
+    def __str__(self):
+        return ('{} : '.format(self.path) +
+                ', '.join('{} {}s'.format(count, _type)
                           for _type, count in self.medias_count.items()))
 
     def __len__(self):
@@ -341,13 +359,14 @@ class Album(UnicodeMixin):
                 root_path = self.path if self.path != '.' else ''
                 if albums_sort_attr.startswith("meta."):
                     meta_key = albums_sort_attr.split(".", 1)[1]
-                    key = lambda s: strxfrm(
+                    key = lambda s: locale.strxfrm(
                         self.gallery.albums[join(root_path, s)].meta.get(meta_key, [''])[0])
                 else:
-                    key = lambda s: strxfrm(getattr(
-                    self.gallery.albums[join(root_path, s)], albums_sort_attr))
+                    key = lambda s: locale.strxfrm(
+                        getattr(self.gallery.albums[join(root_path, s)],
+                                albums_sort_attr))
             else:
-                key = strxfrm
+                key = locale.strxfrm
 
             self.subdirs.sort(key=key,
                               reverse=self.settings['albums_sort_reverse'])
@@ -360,9 +379,9 @@ class Album(UnicodeMixin):
                 key = lambda s: s.date or datetime.now()
             elif medias_sort_attr.startswith('meta.'):
                 meta_key = medias_sort_attr.split(".", 1)[1]
-                key = lambda s: strxfrm(s.meta.get(meta_key, [''])[0])
+                key = lambda s: locale.strxfrm(s.meta.get(meta_key, [''])[0])
             else:
-                key = lambda s: strxfrm(getattr(s, medias_sort_attr))
+                key = lambda s: locale.strxfrm(getattr(s, medias_sort_attr))
 
             self.medias.sort(key=key,
                              reverse=self.settings['medias_sort_reverse'])
@@ -404,16 +423,16 @@ class Album(UnicodeMixin):
 
         if self._thumbnail:
             # stop if it is already set
-            return url_from_path(self._thumbnail)
+            return self._thumbnail
 
         # Test the thumbnail from the Markdown file.
         thumbnail = self.meta.get('thumbnail', [''])[0]
 
         if thumbnail and isfile(join(self.src_path, thumbnail)):
-            self._thumbnail = join(self.name, get_thumb(self.settings,
-                                                        thumbnail))
+            self._thumbnail = url_from_path(join(
+                self.name, get_thumb(self.settings, thumbnail)))
             self.logger.debug("Thumbnail for %r : %s", self, self._thumbnail)
-            return url_from_path(self._thumbnail)
+            return self._thumbnail
         else:
             # find and return the first landscape image
             for f in self.medias:
@@ -426,17 +445,19 @@ class Album(UnicodeMixin):
                         size = get_size(f.src_path)
 
                     if size['width'] > size['height']:
-                        self._thumbnail = join(self.name, f.thumbnail)
+                        self._thumbnail = (url_quote(self.name) + '/' +
+                                           f.thumbnail)
                         self.logger.debug(
-                            "Use 1st landscape image as thumbnail for %r :"
-                            " %s", self, self._thumbnail)
-                        return url_from_path(self._thumbnail)
+                            "Use 1st landscape image as thumbnail for %r : %s",
+                            self, self._thumbnail)
+                        return self._thumbnail
 
             # else simply return the 1st media file
             if not self._thumbnail and self.medias:
                 for media in self.medias:
                     if media.thumbnail is not None:
-                        self._thumbnail = join(self.name, media.thumbnail)
+                        self._thumbnail = (url_quote(self.name) + '/' +
+                                           media.thumbnail)
                         break
                 else:
                     self.logger.warning("No thumbnail found for %r", self)
@@ -444,25 +465,27 @@ class Album(UnicodeMixin):
 
                 self.logger.debug("Use the 1st image as thumbnail for %r : %s",
                                   self, self._thumbnail)
-                return url_from_path(self._thumbnail)
+                return self._thumbnail
 
             # use the thumbnail of their sub-directories
             if not self._thumbnail:
                 for path, album in self.gallery.get_albums(self.path):
                     if album.thumbnail:
-                        self._thumbnail = join(self.name, album.thumbnail)
+                        self._thumbnail = (url_quote(self.name) + '/' +
+                                           album.thumbnail)
                         self.logger.debug(
                             "Using thumbnail from sub-directory for %r : %s",
                             self, self._thumbnail)
-                        return url_from_path(self._thumbnail)
+                        return self._thumbnail
 
         self.logger.error('Thumbnail not found for %r', self)
         return None
 
     @property
     def random_thumbnail(self):
-        try :
-            return url_from_path(join(self.name, random.choice(self.medias).thumbnail))
+        try:
+            return url_from_path(join(self.name,
+                                      random.choice(self.medias).thumbnail))
         except IndexError:
             return self.thumbnail
 
@@ -508,8 +531,10 @@ class Album(UnicodeMixin):
         if zip_gallery and len(self) > 0:
             zip_gallery = zip_gallery.format(album=self)
             archive_path = join(self.dst_path, zip_gallery)
-            if self.settings.get('zip_skip_if_exists', False) and isfile(archive_path):
-                self.logger.debug("Archive %s already created, passing", archive_path)
+            if (self.settings.get('zip_skip_if_exists', False) and
+                    isfile(archive_path)):
+                self.logger.debug("Archive %s already created, passing",
+                                  archive_path)
                 return zip_gallery
 
             archive = zipfile.ZipFile(archive_path, 'w', allowZip64=True)
@@ -587,6 +612,8 @@ class Gallery(object):
                 album.create_output_directories()
                 albums[relpath] = album
 
+        print("\rCollecting albums, done.")
+
         with progressbar(albums.values(), label="%16s" % "Sorting albums",
                          file=self.progressbar_target) as progress_albums:
             for album in progress_albums:
@@ -646,10 +673,7 @@ class Gallery(object):
             # 63 is the total length of progressbar, label, percentage, etc
             available_length = get_terminal_size()[0] - 64
             if x and available_length > 10:
-                text = text_type(x.name)[:available_length]
-                if PY2:
-                    text = text.encode('utf-8')
-                return text
+                return x.name[:available_length]
             else:
                 return ""
 
@@ -697,13 +721,26 @@ class Gallery(object):
             self.remove_files(failed_files)
 
         if self.settings['write_html']:
-            writer = Writer(self.settings, index_title=self.title)
+            album_writer = AlbumPageWriter(self.settings,
+                                           index_title=self.title)
+            album_list_writer = AlbumListPageWriter(self.settings,
+                                                    index_title=self.title)
             with progressbar(self.albums.values(),
                              label="%16s" % "Writing files",
                              item_show_func=log_func, show_eta=False,
                              file=self.progressbar_target) as albums:
                 for album in albums:
-                    writer.write(album)
+                    if album.albums:
+                        if album.medias:
+                            self.logger.warning(
+                                "Album %s contains sub-albums and images. "
+                                "Please move images to their own sub-album. "
+                                "Images in album %s will not be visible.",
+                                album.title, album.title
+                            )
+                        album_list_writer.write(album)
+                    else:
+                        album_writer.write(album)
         print('')
 
         signals.gallery_build.send(self)
